'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = setup;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _hook = require('./hook');

var _hook2 = _interopRequireDefault(_hook);

var _fs = require('fs');

var _path = require('path');

var _utility = require('./utility');

var _lodashAssign = require('lodash.assign');

var _lodashAssign2 = _interopRequireDefault(_lodashAssign);

var _lodashIdentity = require('lodash.identity');

var _lodashIdentity2 = _interopRequireDefault(_lodashIdentity);

var _lodashPick = require('lodash.pick');

var _lodashPick2 = _interopRequireDefault(_lodashPick);

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _postcssModulesExtractImports = require('postcss-modules-extract-imports');

var _postcssModulesExtractImports2 = _interopRequireDefault(_postcssModulesExtractImports);

var _postcssModulesLocalByDefault = require('postcss-modules-local-by-default');

var _postcssModulesLocalByDefault2 = _interopRequireDefault(_postcssModulesLocalByDefault);

var _postcssModulesScope = require('postcss-modules-scope');

var _postcssModulesScope2 = _interopRequireDefault(_postcssModulesScope);

var _parser = require('./parser');

var _parser2 = _interopRequireDefault(_parser);

// cache
var importNr = 0;
var tokensByFile = {};
// processing functions
var preProcess = _lodashIdentity2['default'];
var postProcess = undefined;
// defaults
var lazyResultOpts = {};
var plugins = [_postcssModulesLocalByDefault2['default'], _postcssModulesExtractImports2['default'], _postcssModulesScope2['default']];
var rootDir = process.cwd();

/**
 * @param  {object}   opts
 * @param  {function} opts.createImportedName
 * @param  {function} opts.generateScopedName
 * @param  {function} opts.preprocessCss
 * @param  {function} opts.processCss
 * @param  {string}   opts.rootDir
 * @param  {string}   opts.to
 * @param  {array}    opts.use
 * @param  {array}    opts.extensions
 */

function setup() {
  var opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  // clearing cache
  importNr = 0;
  tokensByFile = {};

  preProcess = (0, _utility.get)('preprocessCss', null, 'function', opts) || _lodashIdentity2['default'];
  postProcess = (0, _utility.get)('processCss', null, 'function', opts) || null;
  rootDir = (0, _utility.get)('rootDir', ['root', 'd'], 'string', opts) || process.cwd();
  // https://github.com/postcss/postcss/blob/master/docs/api.md#processorprocesscss-opts
  lazyResultOpts = (0, _lodashPick2['default'])(opts, ['to']);

  var extraExtensions = (0, _utility.get)('extensions', null, 'array', opts);
  if (extraExtensions) {
    extraExtensions.forEach(function (extension) {
      (0, _hook2['default'])(function (filename) {
        return fetch(filename, filename);
      }, extension);
    });
  }

  // Warning. Options, which aren't affected by plugins, should be processed above.
  var customPlugins = (0, _utility.get)('use', ['u'], 'array', opts);
  if (customPlugins) {
    return void (plugins = customPlugins);
  }

  plugins = [];

  var mode = (0, _utility.get)('mode', null, 'string', opts);
  plugins.push(mode ? new _postcssModulesLocalByDefault2['default']({ mode: opts.mode }) : _postcssModulesLocalByDefault2['default']);

  var createImportedName = (0, _utility.get)('createImportedName', null, 'function', opts);
  plugins.push(createImportedName ? new _postcssModulesExtractImports2['default']({ createImportedName: opts.createImportedName }) : _postcssModulesExtractImports2['default']);

  var generateScopedName = (0, _utility.get)('generateScopedName', null, 'function', opts);
  plugins.push(generateScopedName ? new _postcssModulesScope2['default']({ generateScopedName: opts.generateScopedName }) : _postcssModulesScope2['default']);
}

/**
 * @param  {string} _to    Absolute or relative path. Also can be path to the Node.JS module.
 * @param  {string} _from  Absolute path (relative to root).
 * @param  {string} _trace
 * @return {object}
 */
function fetch(_to, _from, _trace) {
  var trace = _trace || String.fromCharCode(importNr++);
  var newPath = (0, _utility.removeQuotes)(_to);
  // getting absolute path to the processing file
  var filename = /\w/.test(newPath[0]) ? require.resolve(newPath) : (0, _path.resolve)((0, _path.dirname)(_from), newPath);

  // checking cache
  var tokens = tokensByFile[filename];
  if (tokens) {
    return tokens;
  }

  var rootRelativePath = _path.sep + (0, _path.relative)(rootDir, filename);
  var CSSSource = preProcess((0, _fs.readFileSync)(filename, 'utf8'));

  var lazyResult = (0, _postcss2['default'])(plugins.concat(new _parser2['default']({ fetch: fetch, filename: filename, trace: trace }))).process(CSSSource, (0, _lodashAssign2['default'])(lazyResultOpts, { from: rootRelativePath }));

  lazyResult.warnings().forEach(function (message) {
    return console.warn(message.text);
  });

  tokens = lazyResult.root.tokens;
  // updating cache
  tokensByFile[filename] = tokens;

  if (postProcess) {
    postProcess(lazyResult.css);
  }

  return tokens;
}

(0, _hook2['default'])(function (filename) {
  return fetch(filename, filename);
}, '.css');
module.exports = exports['default'];