'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.get = get;
exports.is = is;
exports.removeQuotes = removeQuotes;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodashIsarray = require('lodash.isarray');

var _lodashIsarray2 = _interopRequireDefault(_lodashIsarray);

var _lodashIsfunction = require('lodash.isfunction');

var _lodashIsfunction2 = _interopRequireDefault(_lodashIsfunction);

var _lodashIsstring = require('lodash.isstring');

var _lodashIsstring2 = _interopRequireDefault(_lodashIsstring);

var _util = require('util');

var check = {
  'array': _lodashIsarray2['default'],
  'function': _lodashIsfunction2['default'],
  'string': _lodashIsstring2['default']
};

/**
 * @param  {string} prop
 * @param  {string[]} aliases
 * @param  {string} type
 * @param  {object} source
 * @return {*}
 */

function get(prop, aliases, type, source) {
  if (source[prop]) {
    if (!is(type, source[prop])) {
      throw new Error((0, _util.format)('should specify %s for %s', type, prop));
    }

    return source[prop];
  }

  if (!(0, _lodashIsarray2['default'])(aliases)) {
    return null;
  }

  var deprecatedProp = undefined;
  var match = aliases.some(function (alias) {
    return Boolean(source[deprecatedProp = alias]);
  });

  if (match) {
    if (!is(type, source[deprecatedProp])) {
      throw new Error((0, _util.format)('should specify %s for %s', type, deprecatedProp));
    }

    // deprecated message
    return source[deprecatedProp];
  }

  return null;
}

/**
 * @param  {string}  type
 * @param  {*}       value
 * @return {boolean}
 */

function is(type, value) {
  return check[type](value);
}

/**
 * @param  {string} str
 * @return {string}
 */

function removeQuotes(str) {
  return str.replace(/^["']|["']$/g, '');
}